/* Ax93304 board API Library */

#include <stdio.h>
#include <unistd.h>
#include <sys/io.h>			//for kernel 2.6
#include <curses.h>

/* UART address */
#define Com1    0x3f8   //COM1 address
#define Com2    0x2f8   //COM2 address
#define Com3    0x3e8   //COM3 address
#define Com4    0x3e8   //COM4 address
#define Rx  0           //Receive buffer (RD)
#define Tx  0           //Transmit buffer (WR)
#define DLL 0           //Baud rate divisor LSB (WR)
#define DLM 1           //Baud rate divisor MSB (WR)
#define IER 1           //Interrupt enable register (RD)
#define ISR 2           //Interrupt status register (RD)
#define FCR 2           //FIFO control register (WR)
#define UCR 3           //UART control register (R/W)
#define HCR 4           //Handshake control rigister (R/W)
#define USR 5           //UART status register (R/W)
#define HSR 6           //Handshake status register (R/W)

//UART Command
#define MBR     11520       //baud rate = 115200 bps
#define FEN     1           //FIFO enable
#define FDIS    0           //FIFO disable
#define IDIS    0           //All intr. disable
#define IHEN    0x08        //Intr. on handshake input state change
#define IEEN    0x04        //Intr. on parity,overrun,framing error or break
#define ITEN    0x02        //Intr. on transmit buffer empty
#define IREN    0x01        //Intr. on receive data ready
#define DSTR    0x00        //Data latch select Tx/Rx data buffer register
#define DSBR    0x80        //Data latch select baud rate generator divisor
#define BKOFF   0           //Break condition off
#define BKON    0x40        //Break condition on
#define PRTNA   0           //Parity none
#define PRTOD   0x08        //Parity odd
#define PRTEV   0x18        //parity even
#define PRTMK   0x28        //parity mark
#define PRTSP   0x38        //Parity space
#define STB1    0           //Stop bit 1
#define STB2    0x04        //Stop 2 bits
#define DB5     0           //Data 5 bits
#define DB6     0x01        //Data 6 bits
#define DB7     0x02        //Data 7 bits
#define DB8     0x03        //Data 8 bits
#define TxBE    0x20        //Transmit buffer empty
#define RxDR    0x01        //Receive data ready
#define Timeout 0xffff      //UART timer out


unsigned int	Bport;

//int _Tx_data(char txd);

int _init_com(int ccom, unsigned long baud, int D_bits, int S_bits, int Prty) {
    unsigned int dvs;
    unsigned char urtc=0;

    switch(ccom){	    //set base address
	case 1:
	    Bport = Com1;break;
	case 2:
	    Bport = Com2;break;
	case 3:
	    Bport = Com3;break;
	case 4:
	    Bport = Com4;break;
	default:
	    return 1;
    }

    switch(baud){
	case 1200:break;
	case 2400:break;
	case 4800:break;
	case 9600:break;
	case 19200:break;
	case 38400:break;
	case 57600:break;
	case 115200:break;
	default:return 5;
    }


    ioperm(Bport, 7, 1);
    outb(IDIS, Bport+IER);	//disable all intr.
    outb(FDIS, Bport+FCR);	//disable FIFO
    dvs=MBR/(baud/10);          //set baud rate divisor
    outb(DSBR, Bport+UCR);
    outb(dvs&0x00ff, Bport+DLL);
    outb(dvs>>8, Bport+DLM);

    switch(D_bits){		//set uart control register
	case 5:
	    urtc = urtc | DB5;break;
	case 6:
	    urtc = urtc | DB6;break;
	case 7:
	    urtc = urtc | DB7;break;
	case 8:
	    urtc = urtc | DB8;break;
	default:
	    return 2;
    }

    switch(S_bits){
	case 1:
            urtc = urtc | STB1;break;
        case 2:
            urtc = urtc | STB2;break;
        default:
            return 3;
    }

    switch(Prty){
        case 0:
            urtc = urtc | PRTNA;break;
        case 1:
            urtc = urtc | PRTOD;break;
        case 2:
            urtc = urtc | PRTEV;break;
        case 3:
            urtc = urtc | PRTMK;break;
        case 4:
            urtc = urtc | PRTSP;break;
        default:
            return 4;
    }
    outb(urtc, Bport+UCR);

    outb(0x03, Bport+FCR);      //clear FIFO
    outb(FEN, Bport+FCR);
    ioperm(Bport, 7, 0);

    printf("\tCOM PORT = %1d \t   IO ADDRESS = %3X \t  BAUD RATE = %6ld",ccom,Bport,baud);
    return 0;
}

/*
int _show_lcm(char *_str){
    int ii;
    int err=0;

    _Tx_data(0X0D);
//	usleep(1000000);  //1 sec
    for (ii = 0;ii < 80;ii ++){
        err = _Tx_data(*(_str+ii));
        if (ii < 16){
            printf("%c",*(_str+ii));
        }
        if ((ii >= 40) && (ii < 56)){
            printf("%c",*(_str+ii));
        }
        if (err == 1) return(err);
    }
    return(err);
}

int _Return(void) {
    _Tx_data(0xFE);
    _Tx_data(0x02);
    return 0;
}

int _Display_OnOff(int display, int cursor, int blink) {
    _Tx_data(0xFE);
    if(display==0) {
	if(cursor==0) {
	    if(blink==0)    _Tx_data(0x00);
	    else    _Tx_data(0x01);
	}
	else {
	    if(blink==0)    _Tx_data(0x02);
	    else    _Tx_data(0x03);
	}
    }
    else {
	if(cursor==0) {
	    if(blink==0)    _Tx_data(0x04);
	    else    _Tx_data(0x05);
	}
	else {
	    if(blink==0)    _Tx_data(0x06);
	    else    _Tx_data(0x07);
	}
    }
    return 0;
}

int _CurDis_shift(int cursor, int display) {
    _Tx_data(0xFE);
    if(cursor==0) {
	if(display==0)	_Tx_data(0x00);
	else	_Tx_data(0x04);
    }
    else {
	if(display==0)	_Tx_data(0x08);
	else	_Tx_data(0x0C);
    }
    return 0;
}

int _up_down(int flag) {

    if(flag==0)	_Tx_data(0x00); //up
    else _Tx_data(0x01); //down

    return 0;
}
*/

char _Rx_key(void){
    char _c = 0;
    ioperm(Bport, 7, 1);

    if((inb(Bport+USR) & RxDR) == RxDR)
	_c = inb(Bport+Rx);

    ioperm(Bport, 7, 0);
    return _c;
}

int _Tx_data(char txd){
    unsigned int ii=0;

    ioperm(Bport, 7, 1);

    while((inb(Bport+USR) & TxBE) == 0){
        usleep(10000);
        ii++;
        if (ii == 65535)
            return 1;
    }
    outb(txd, Bport+Tx);
    while((inb(Bport+USR) & TxBE) == 0){
        usleep(10000);
        ii++;
        if (ii == 65535)
            return 1;
    }
    ii=0;

    ioperm(Bport, 7, 0);
    return 0;
}


